import os
import re

from quixote import get_publisher
from qommon.storage import StorableObject

from apps import App


def reload_apache():
    try:
        os.system('/usr/sbin/larpe-reload-apache2')
    except:
        pass

class Site(StorableObject):
    _names = 'sites'

    name = None
    admin_email = None
    hostname = None
    app_id = None

    def store(self):
        old_ver = None
        if self.id:
            try:
                old_ver = self.get(self.id)
            except:
                pass

        StorableObject.store(self)

        if old_ver and self.hostname != old_ver.hostname:
            # old version used a different hostname; remove old conf
            old_ver.remove_extras()

        self.export_to_apache_vhost()

    def get_app(self):
        if self.app_id:
            try:
                return App.get(self.app_id)
            except KeyError:
                return None
        else:
            return None

    def set_app(self, app):
        if app:
            self.app_id = app.id
        elif self.app_id:
            self.app_id = None
    app = property(get_app, set_app)

    def export_to_apache_vhost(self):
        template = self.app.get_vhost_template()
        self.vhost_logs_dir = os.path.join(get_publisher().app_dir, 'vhosts_logs', self.hostname)
        vhost = template % self.__dict__

        if not os.path.exists(os.path.join(get_publisher().app_dir, 'vhosts.d')):
            os.mkdir(os.path.join(get_publisher().app_dir, 'vhosts.d'))

        apache_conf = os.path.join(get_publisher().app_dir, 'vhosts.d',
                '%s-apache.conf' % self.hostname)
        file(apache_conf, 'w').write(vhost)

        if not os.path.exists(os.path.join(get_publisher().app_dir, 'vhosts_logs')):
            os.mkdir(os.path.join(get_publisher().app_dir, 'vhosts_logs'))
        if not os.path.exists(self.vhost_logs_dir):
            os.mkdir(self.vhost_logs_dir)

        for include in re.findall('[Ii]nclude (.*)', vhost):
            if '?' in include or '*' in include:
                continue # ignore wildcards
            if os.path.exists(include):
                continue
            if not os.path.exists(os.path.dirname(include)):
                os.makedirs(os.path.dirname(include))
            file(include, 'w').close()

        reload_apache()


    def remove_object(cls, id):
        object = cls.get(id)
        object.remove_extras()
        super(Site, cls).remove_object(id)
        reload_apache()
    remove_object = classmethod(remove_object)

    def remove_extras(self):
        apache_conf = os.path.join(get_publisher().app_dir, 'vhosts.d',
                '%s-apache.conf' % self.hostname)
        if os.path.exists(apache_conf):
            os.unlink(apache_conf)
        
        vhost_logs_dir = os.path.join(get_publisher().app_dir, 'vhosts_logs', self.hostname)
        if os.path.exists(vhost_logs_dir):
            if len(os.listdir(vhost_logs_dir)) == 0:
                # no file, log dir can be removed safely
                os.rmdir(vhost_logs_dir)

